import 'package:contentsquare/src/modules/screengraph/screengraph_node.dart';
import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:webview_flutter/webview_flutter.dart';

class MockRootWidget extends StatelessWidget {
  final Widget child;

  const MockRootWidget({Key? key, required this.child}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Center(child: child);
  }
}

ScreengraphNode createRootElementNode({required List<ScreengraphNode> children}) {
  final rootElement = find.byType(MockRootWidget).first.evaluate().elementAt(0);
  return ScreengraphNode(element: rootElement, index: 0, children: children);
}

void main() {
  group('toJson', () {
    testWidgets(
        'Should return the JSON formatted Map of the Screengraph node corresponding to the given element',
        (WidgetTester tester) async {
      await tester.pumpWidget(
        const MockRootWidget(
            child: Text(
          'Hello',
          key: Key('hello_text'),
          textDirection: TextDirection.ltr,
        )),
      );

      final element =
          find.byKey(const Key('hello_text')).first.evaluate().elementAt(0);
      final node = ScreengraphNode(
        element: element,
        index: 0,
        children: [],
      );
       createRootElementNode(children: [node]);

      var generatedJson = await node.toJson(pixelRatio: 1.0, rootPath: 'MockRootPath');

      var expectedJson = {
        'id': '[<\'hello_text\'>]',
        'metadata': {
          'class_name': 'Text',
          'fullpath': 'MockRootPath>MockRootWidget:eq(0)>Text:eq(0)',
        },
        'recordingId': generatedJson['recordingId'],
        'style': {
          'height': 14.0,
          'width': 70.0,
          'x': 365.0,
          'y': 293.0,
          'visibility': true,
          'bg': '#00FFFFFF',
        },
        'format': 1,
        'children': []
      };
      expect(await node.toJson(pixelRatio: 1.0, rootPath: 'MockRootPath'),
          expectedJson);
    });

    testWidgets(
        'Should return JSON with "format": 2 when a webview is present',
        (WidgetTester tester) async {
      await tester.pumpWidget(
        const MockRootWidget(
            child: WebView(initialUrl: 'https://contentsquare.com/')),
      );

      final element = find.byType(WebView).first.evaluate().elementAt(0);
      final node = ScreengraphNode(
        element: element,
        index: 0,
        children: []
      );
      createRootElementNode(children: [node]);

      final result = await (node.toJson(pixelRatio: 1.0, rootPath: 'MockRootPath'));
      expect(result['metadata']['fullpath'],
          'MockRootPath>MockRootWidget:eq(0)>WebView:eq(0)');
      expect(result['format'], 2);
    });

    testWidgets(
        'Should return a "bg" field with the default color if element is not a RepaintBoundary',
        (WidgetTester tester) async {
      await tester.pumpWidget(
        const MockRootWidget(
          child: Text(
            'Hello',
            key: Key('hello_text'),
            textDirection: TextDirection.ltr,
          ),
        ),
      );

      final element =
          find.byKey(const Key('hello_text')).first.evaluate().elementAt(0);
      final node = ScreengraphNode(
        element: element,
        index: 0,
        children: []
      );
      createRootElementNode(children: [node]);

      final result = await (node.toJson(pixelRatio: 1.0, rootPath: 'MockRootPath'));
      expect(result['style']['bmp'], null);
      expect(result['style']['bg'], '#00FFFFFF');
    });
  });
}
